@@ -19,12 +19,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SeahorseKeyStore: A GtkTreeStore for displaying keys in a table. 
+ * 
+ * - Gets its keys from a SeahorseKeyset.
+ * - Allows filtering by setting the "mode" and "filter" properties.
+ * - Sorts keys. 
+ * 
+ * Properties:
+ *   mode: (SeahorseKeyStoreMode) Setup the filtering mode. 
+ *   filter: (gchar*) Set the filter text for KEY_STORE_MODE_FILTERED. 
+ */
+ 
 #ifndef __SEAHORSE_KEY_STORE_H__
 #define __SEAHORSE_KEY_STORE_H__
 
 #include <gtk/gtk.h>
 
 #include "seahorse-context.h"
+#include "seahorse-keyset.h"
 
 #define SEAHORSE_TYPE_KEY_STORE			(seahorse_key_store_get_type ())
 #define SEAHORSE_KEY_STORE(obj)			(GTK_CHECK_CAST ((obj), SEAHORSE_TYPE_KEY_STORE, SeahorseKeyStore))
@@ -47,7 +60,7 @@ struct _SeahorseKeyStore {
 	GtkTreeStore		    parent;
  
     /*< public >*/
-	SeahorseKeySource	    *sksrc;
+    SeahorseKeyset          *skset;
     
     /*< private >*/
     SeahorseKeyStorePriv    *priv;
@@ -97,7 +110,7 @@ enum {
     KEY_STORE_DATA,
     KEY_STORE_CHECK,
     KEY_STORE_PAIR,
-    KEY_STORE_NOTPAIR,
+    KEY_STORE_STOCK_ID,
     KEY_STORE_NAME,
     KEY_STORE_KEYID,
     KEY_STORE_UID,
@@ -123,20 +136,20 @@ enum {
             G_TYPE_POINTER,  \
             G_TYPE_BOOLEAN,  \
             G_TYPE_BOOLEAN,  \
-            G_TYPE_BOOLEAN,  \
+            G_TYPE_STRING,   \
             G_TYPE_STRING,   \
             G_TYPE_STRING,   \
             G_TYPE_INT
             
 
-GType           seahorse_key_store_get_type ();
+GType               seahorse_key_store_get_type             ();
 
-void			seahorse_key_store_init			(SeahorseKeyStore	*skstore,
-								 GtkTreeView		*view);
+void                seahorse_key_store_init                 (SeahorseKeyStore   *skstore,
+								                             GtkTreeView        *view);
 
-void			seahorse_key_store_populate		(SeahorseKeyStore	*skstore);
+void                seahorse_key_store_populate             (SeahorseKeyStore   *skstore);
 
-SeahorseKey*		seahorse_key_store_get_key_from_path	(GtkTreeView		*view,
+SeahorseKey*	    seahorse_key_store_get_key_from_path	(GtkTreeView		*view,
                                                              GtkTreePath		*path,
                                                              guint              *uid);
 
@@ -144,6 +157,8 @@ GtkTreeViewColumn*	seahorse_key_store_append_column	(GtkTreeView		*view,
 								 const gchar		*name,
 								 const gint		index);
 
+GList*              seahorse_key_store_get_all_keys         (GtkTreeView        *view);
+
 GList*              seahorse_key_store_get_selected_keys    (GtkTreeView        *view);
 
 SeahorseKey*        seahorse_key_store_get_selected_key	    (GtkTreeView		*view,
