@@ -29,9 +29,12 @@
 #include "seahorse-keyserver-control.h"
 #include "seahorse-server-source.h"
 #include "seahorse-gconf.h"
+#include "seahorse-pgp-key.h"
 
 /* From seahorse_preferences_cache.c */
-void seahorse_prefs_cache (SeahorseContext *ctx, SeahorseWidget *widget);
+void seahorse_prefs_cache (SeahorseWidget *widget);
+
+/* Key Server Prefs --------------------------------------------------------- */
 
 #ifdef WITH_KEYSERVER
 
@@ -50,13 +53,16 @@ keyserver_cell_edited (GtkCellRendererText *cell, gchar *path, gchar *text,
                        GtkTreeModel *model)
 {
     GtkTreeIter iter;
+    gboolean ret;
     
     if (!seahorse_server_source_valid_uri (text)) {
-        seahorse_util_show_error (NULL, _("<b><big>Not a valid Key Server address.</big></b>\n\nFor help contact your system adminstrator or the administrator of the key server." ));
+        seahorse_util_show_error (NULL, _("Not a valid Key Server address."), 
+                                  _("For help contact your system adminstrator or the administrator of the key server." ));
         return;
     }
     
-    g_return_if_fail (gtk_tree_model_get_iter_from_string (model, &iter, path));
+    ret = gtk_tree_model_get_iter_from_string (model, &iter, path);
+    g_assert (ret);
     gtk_tree_store_set (GTK_TREE_STORE (model), &iter, KEYSERVER_COLUMN, text, -1);
 }
 
@@ -332,8 +338,8 @@ keyserver_add_clicked (GtkButton *button, SeahorseWidget *sw)
     GtkTreeStore *store;
     GtkTreeIter iter;
     
-    swidget = seahorse_widget_new_allow_multiple ("add-keyserver", sw->sctx);
-	g_return_if_fail (swidget != NULL);
+    swidget = seahorse_widget_new_allow_multiple ("add-keyserver");
+    g_return_if_fail (swidget != NULL);
     
     widget = glade_xml_get_widget (swidget->xml, "keyserver-type");
     g_return_if_fail (widget != NULL);
@@ -379,7 +385,7 @@ keyserver_add_clicked (GtkButton *button, SeahorseWidget *sw)
 
 /* Perform keyserver page initialization */
 static void
-setup_keyservers (SeahorseContext *sctx, SeahorseWidget *swidget)
+setup_keyservers (SeahorseWidget *swidget)
 {
     GtkTreeView *treeview;
     SeahorseKeyserverControl *skc;
@@ -423,19 +429,37 @@ setup_keyservers (SeahorseContext *sctx, SeahorseWidget *swidget)
 
 #endif /* WITH_KEYSERVER */
 
+#ifdef WITH_SHARING 
+
+/* Key Sharing Prefs -------------------------------------------------------- */
+
 static void
-default_key_changed (SeahorseDefaultKeyControl *sdkc, gpointer *data)
+setup_sharing (SeahorseWidget *swidget)
 {
-    const gchar *id = seahorse_default_key_control_active_id (sdkc);
+    GtkWidget *widget;
+    widget = glade_xml_get_widget (swidget->xml, "enable-sharing-placeholder");
+    gtk_container_add (GTK_CONTAINER (widget),
+        seahorse_check_button_control_new (_("_Share my keys with others on my network"), KEYSHARING_KEY));
+    gtk_widget_show_all (widget);
+}
+
+#endif /* WITH_SHARING */
+
+/* -------------------------------------------------------------------------- */
+
+static void
+default_key_changed (GtkOptionMenu *combo, gpointer *data)
+{
+    const gchar *id = seahorse_combo_keys_get_active_id (combo);
     seahorse_gconf_set_string (DEFAULT_KEY, id == NULL ? "" : id);    
 }
 
 static void
 gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, 
-                    SeahorseDefaultKeyControl *sdkc)
+                    GtkOptionMenu *combo)
 {
     const gchar *key_id = gconf_value_get_string (gconf_entry_get_value (entry));
-    seahorse_default_key_control_select_id (sdkc, key_id);
+    seahorse_combo_keys_set_active_id (combo, key_id);
 }
 
 static void
@@ -447,22 +471,20 @@ remove_gconf_notification (GObject *obj, gpointer data)
 
 /**
  * seahorse_prefs_new
- * @sctx: The SeahorseContext to work with
  * 
  * Create a new preferences window.
  * 
  * Returns: The preferences window.
  **/
 SeahorseWidget *
-seahorse_prefs_new (SeahorseContext *sctx)
+seahorse_prefs_new ()
 {
+    SeahorseKeyset *skset;
     SeahorseWidget *swidget;
     GtkWidget *widget;
-    SeahorseDefaultKeyControl *sdkc;
-    guint removed = 0;
     guint gconf_id;
     
-    swidget = seahorse_widget_new ("prefs", sctx);
+    swidget = seahorse_widget_new ("prefs");
     
     widget = glade_xml_get_widget (swidget->xml, "modes");
     gtk_container_add (GTK_CONTAINER (widget),
@@ -471,38 +493,47 @@ seahorse_prefs_new (SeahorseContext *sctx)
         seahorse_check_button_control_new (_("_Armor Encode Files"), ARMOR_KEY));
     gtk_widget_show_all (widget);
 
-    widget = glade_xml_get_widget (swidget->xml, "default_key");
+    widget = glade_xml_get_widget (swidget->xml, "signer-select");
+    g_return_val_if_fail (widget != NULL, NULL);
 
-    sdkc = seahorse_default_key_control_new (seahorse_context_get_key_source (sctx), 
-                                             _("None. Prompt for a key."));
-    gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (sdkc));
-    gtk_widget_show_all (widget);
+    skset = seahorse_keyset_new (SKEY_PGP, 
+                                 SKEY_PRIVATE, 
+                                 SKEY_LOC_LOCAL, 
+                                 SKEY_FLAG_CAN_SIGN, 
+                                 SKEY_FLAG_EXPIRED | SKEY_FLAG_REVOKED | SKEY_FLAG_DISABLED);
+    seahorse_combo_keys_attach (GTK_OPTION_MENU (widget), skset, _("None. Prompt for a key."));
+    g_object_unref (skset);
+    
+    seahorse_combo_keys_set_active_id (GTK_OPTION_MENU (widget), seahorse_gconf_get_string (DEFAULT_KEY));    
+    g_signal_connect (widget, "changed", G_CALLBACK (default_key_changed), NULL);
 
-    seahorse_default_key_control_select_id (sdkc, seahorse_gconf_get_string (DEFAULT_KEY));    
-    g_signal_connect (sdkc, "changed", G_CALLBACK (default_key_changed), NULL);
-
-    gconf_id = seahorse_gconf_notify (DEFAULT_KEY, (GConfClientNotifyFunc)gconf_notification, sdkc);
-    g_signal_connect (sdkc, "destroy", G_CALLBACK (remove_gconf_notification), GINT_TO_POINTER (gconf_id));
+    gconf_id = seahorse_gconf_notify (DEFAULT_KEY, (GConfClientNotifyFunc)gconf_notification, GTK_OPTION_MENU (widget));
+    g_signal_connect (widget, "destroy", G_CALLBACK (remove_gconf_notification), GINT_TO_POINTER (gconf_id));
     
 #ifdef WITH_AGENT   
-    seahorse_prefs_cache (sctx, swidget);
+    seahorse_prefs_cache (swidget);
 #else
-    widget = glade_xml_get_widget (swidget->xml, "notebook");
-    gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 1 - removed);
-    removed++;
+    widget = glade_xml_get_widget (swidget->xml, "cache-tab");
+    g_return_val_if_fail (GTK_IS_WIDGET (widget), swidget);
+    seahorse_prefs_remove_tab (swidget, widget);
 #endif
 
 #ifdef WITH_KEYSERVER
-    setup_keyservers (sctx, swidget);
+    setup_keyservers (swidget);
 #else
-    widget = glade_xml_get_widget (swidget->xml, "notebook");
-    gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2 - removed);
-    removed++;
+    widget = glade_xml_get_widget (swidget->xml, "keyserver-tab");
+    g_return_val_if_fail (GTK_IS_WIDGET (widget), swidget);
+    seahorse_prefs_remove_tab (swidget, widget);
 #endif
 
-    /* To suppress warnings */
-    removed = 0;
-    
+#ifdef WITH_SHARING
+    setup_sharing (swidget);
+#else
+    widget = glade_xml_get_widget (swidget->xml, "sharing-tab");
+    g_return_val_if_fail (GTK_IS_WIDGET (widget), swidget);
+    seahorse_prefs_remove_tab (swidget, widget);
+#endif    
+
     seahorse_widget_show (swidget);
     return swidget;
 }
@@ -523,8 +554,33 @@ seahorse_prefs_add_tab (SeahorseWidget *swidget, GtkWidget *label, GtkWidget *ta
 }
 
 void                
-seahorse_prefs_select_tab   (SeahorseWidget *swidget, guint tab)
+seahorse_prefs_select_tab (SeahorseWidget *swidget, GtkWidget *tab)
 {
-    GtkWidget *widget = glade_xml_get_widget (swidget->xml, "notebook");
-    gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), (gint)tab);
+    GtkWidget *tabs;
+    gint pos;
+    
+    g_return_if_fail (GTK_IS_WIDGET (tab));
+    
+    tabs = glade_xml_get_widget (swidget->xml, "notebook");
+    g_return_if_fail (GTK_IS_NOTEBOOK (tabs));
+    
+    pos = gtk_notebook_page_num (GTK_NOTEBOOK (tabs), tab);
+    if (pos != -1)
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (tabs), pos);
 }    
+
+void 
+seahorse_prefs_remove_tab (SeahorseWidget *swidget, GtkWidget *tab)
+{
+    GtkWidget *tabs;
+    gint pos;
+    
+    g_return_if_fail (GTK_IS_WIDGET (tab));
+    
+    tabs = glade_xml_get_widget (swidget->xml, "notebook");
+    g_return_if_fail (GTK_IS_NOTEBOOK (tabs));
+    
+    pos = gtk_notebook_page_num (GTK_NOTEBOOK (tabs), tab);
+    if (pos != -1)
+        gtk_notebook_remove_page (GTK_NOTEBOOK (tabs), pos);
+}
