@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
+ * Copyright (C) 2004,2005 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +31,10 @@
 #include "seahorse-op.h"
 #include "seahorse-util.h"
 #include "seahorse-libdialogs.h"
+#include "seahorse-pgp-source.h"
+#include "seahorse-pgp-key.h"
+#include "seahorse-gtkstock.h"
+#include "seahorse-secure-memory.h"
 
 typedef enum _CmdLineMode {
     MODE_NONE,
@@ -66,7 +71,9 @@ static const struct poptOption options[] = {
     { "uri-list", 'T', POPT_ARG_NONE | POPT_ARG_VAL, &read_uris, TRUE,
       N_("Read list of URIs on standard in"), NULL },
 	
-	{ NULL, '\0', 0, NULL, 0, NULL, NULL }
+	POPT_AUTOHELP
+	
+	POPT_TABLEEND
 };
 
 
@@ -122,9 +129,9 @@ read_uri_arguments (GnomeProgram *program)
 
 /* Perform an import on the given set of paths */
 static guint
-do_import (SeahorseContext *sctx, const gchar **paths)
+do_import (const gchar **paths)
 {
-    SeahorseKeySource *sksrc;
+    SeahorsePGPSource *psrc;
     GtkWidget *dlg;
     GError *err = NULL;
     gint keys = 0;
@@ -133,15 +140,15 @@ do_import (SeahorseContext *sctx, const gchar **paths)
     gchar *t;
     guint ret = 0;
     
-    sksrc = seahorse_context_get_key_source (sctx);
-    g_return_val_if_fail (sksrc != NULL, 1);
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL));
+    g_return_val_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc), 1);
     
     /* Get all sub-folders etc... */
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
      
     for (u = uris; *u; u++) {
-        keys += seahorse_op_import_file (sksrc, *u, &err);
+        keys += seahorse_op_import_file (psrc, *u, &err);
        
         if (err != NULL) {
             seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""),
@@ -156,6 +163,7 @@ do_import (SeahorseContext *sctx, const gchar **paths)
                                     _("Imported %d keys"), keys);
         dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                              GTK_MESSAGE_INFO, GTK_BUTTONS_OK, t);
+        g_free (t);
         gtk_dialog_run (GTK_DIALOG (dlg));
         gtk_widget_destroy (dlg);
     }
@@ -166,32 +174,28 @@ do_import (SeahorseContext *sctx, const gchar **paths)
 
 /* Encrypt or sign the given set of paths */
 static guint 
-do_encrypt_base (SeahorseContext *sctx, const gchar **paths, gboolean sign)
+do_encrypt_base (const gchar **paths, gboolean sign)
 {
-    SeahorseKeyPair *signer = NULL;
-    SeahorseKeySource *sksrc;
+    SeahorsePGPKey *signer = NULL;
     GList *keys = NULL;
     gchar *new_path;
     gpgme_error_t err;
     gchar **uris = NULL;
     gchar **u;
     guint ret = 0;
-
-    sksrc = seahorse_context_get_key_source (sctx);
-    g_return_val_if_fail (sksrc != NULL, 1);
     
     /* This can change the list of uris and optionally compress etc... */
-    uris = seahorse_process_multiple (sctx, paths, NULL);
+    uris = seahorse_process_multiple (paths, NULL);
 
     if(!uris)
         ret = 1;
     else {
-        keys = seahorse_recipients_get (sctx, sign ? &signer : NULL);
+        keys = seahorse_recipients_get (sign ? &signer : NULL);
            
         if (g_list_length (keys) > 0) {
             for(u = uris; *u; u++) {
                
-                new_path = seahorse_util_add_suffix (sksrc->ctx, *u, SEAHORSE_CRYPT_SUFFIX, 
+                new_path = seahorse_util_add_suffix (*u, SEAHORSE_CRYPT_SUFFIX, 
                                                      _("Choose Encrypted File Name for '%s'"));
                 if (!new_path) 
                     break;
@@ -222,33 +226,29 @@ do_encrypt_base (SeahorseContext *sctx, const gchar **paths, gboolean sign)
 
 /* Encrypt the given set of paths */
 static guint
-do_encrypt (SeahorseContext *sctx, const gchar **paths)
+do_encrypt (const gchar **paths)
 {
-    return do_encrypt_base (sctx, paths, FALSE);
+    return do_encrypt_base (paths, FALSE);
 }
 
 /* Encrypt and sign the given set of paths */
 static guint
-do_encrypt_sign (SeahorseContext *sctx, const gchar **paths)
+do_encrypt_sign (const gchar **paths)
 {
-    return do_encrypt_base (sctx, paths, TRUE);
+    return do_encrypt_base (paths, TRUE);
 }
 
 static guint
-do_sign (SeahorseContext *sctx, const gchar **paths)
+do_sign (const gchar **paths)
 {
-    SeahorseKeySource *sksrc;
-    SeahorseKeyPair *signer;
+    SeahorsePGPKey *signer;
     gpgme_error_t err;
     gchar **uris;
     gchar **u;
     gchar *new_path;
     guint ret = 0;
     
-    sksrc = seahorse_context_get_key_source (sctx);
-    g_return_val_if_fail (sksrc != NULL, 1);
-
-    signer = seahorse_signer_get (sctx);    
+    signer = seahorse_signer_get ();    
     if (signer == NULL)
         return 1;
     
@@ -257,7 +257,7 @@ do_sign (SeahorseContext *sctx, const gchar **paths)
      
     for (u = uris; *u; u++) {
       
-        new_path = seahorse_util_add_suffix (sksrc->ctx, *u, SEAHORSE_SIG_SUFFIX, 
+        new_path = seahorse_util_add_suffix (*u, SEAHORSE_SIG_SUFFIX, 
                                              _("Choose Signature File Name for '%s'"));
         if (!new_path)
             break;
@@ -279,19 +279,18 @@ do_sign (SeahorseContext *sctx, const gchar **paths)
 
 /* Decrypt the given set of paths */
 static guint
-do_decrypt (SeahorseContext *sctx, const gchar **paths)
+do_decrypt (const gchar **paths)
 {
-    SeahorseKeySource *sksrc;
+    SeahorsePGPSource *psrc;
     gpgme_verify_result_t status = NULL;
-    SeahorseWidget *signatures = NULL;
     gpgme_error_t err;
     gchar **uris;
     gchar **u;
     gchar *new_path;
     guint ret = 0;
-	
-    sksrc = seahorse_context_get_key_source (sctx);
-    g_return_val_if_fail (sksrc != NULL, 1);
+
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL));
+    g_return_val_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc), 1);
         
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
@@ -302,7 +301,7 @@ do_decrypt (SeahorseContext *sctx, const gchar **paths)
         if(!new_path)
             break;
             
-        seahorse_op_decrypt_verify_file (sksrc, *u, new_path, &status, &err);
+        seahorse_op_decrypt_verify_file (psrc, *u, new_path, &status, &err);
                 
         if (!GPG_IS_OK (err)) {
             seahorse_util_handle_gpgme (err, _("Couldn't decrypt \"%s\""),
@@ -312,29 +311,23 @@ do_decrypt (SeahorseContext *sctx, const gchar **paths)
         }
     
         if(status && status->signatures) {
-            if(!signatures)
-                signatures = seahorse_signatures_new (sctx);
-            seahorse_signatures_add (sctx, signatures, new_path, status);
+            seahorse_signatures_notify (new_path, status);
         }
 
         g_free (new_path);
     }
     
     g_strfreev (uris);    
-    
-    if (ret == 0 && signatures)
-        seahorse_signatures_run (sctx, signatures);
         
     return ret;
 }
 
 /* Verify the given set of paths. Prompt user if original files not found */
 static guint
-do_verify (SeahorseContext *sctx, const gchar **paths)
+do_verify (const gchar **paths)
 {
-    SeahorseKeySource *sksrc;
+    SeahorsePGPSource *psrc;
     gpgme_verify_result_t status = NULL;
-    SeahorseWidget *signatures = NULL;
     gchar *original = NULL;
     gpgme_error_t err;
     gchar **uris;
@@ -343,9 +336,9 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
     
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
-	
-    sksrc = seahorse_context_get_key_source (sctx);
-    g_return_val_if_fail (sksrc != NULL, 1);
+
+    psrc = SEAHORSE_PGP_SOURCE (seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL));
+    g_return_val_if_fail (psrc != NULL && SEAHORSE_IS_PGP_SOURCE (psrc), 1);
          
     for (u = uris; *u; u++) {
 
@@ -381,7 +374,13 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
         }
         
         if (original) {
-            seahorse_op_verify_file (sksrc, *u, original, &status, &err);
+            seahorse_op_verify_file (psrc, *u, original, &status, &err);
+
+            if (GPG_IS_OK (err)) {
+                if (status && status->signatures)
+                    seahorse_signatures_notify (original, status);
+            }
+            
             g_free (original);
 
             if (!GPG_IS_OK (err)) {
@@ -391,54 +390,86 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
                 break;
             }
             
-            if (status && status->signatures) {
-                if(!signatures)
-                    signatures = seahorse_signatures_new (sctx);
-                seahorse_signatures_add (sctx, signatures, *u, status);
-            }
         }
     }
     
     g_strfreev (uris);    
     
-    if (ret == 0 && signatures)
-        seahorse_signatures_run (sctx, signatures);
-        
     return ret;
 }
 
+/* Checks to see if any dialogs are open when we're called  
+   as a command line app. Once all are gone, closes app */
+static gboolean
+check_dialogs (gpointer dummy)
+{
+    if(seahorse_notification_have ())
+        return TRUE;
+
+    gtk_main_quit ();
+    return FALSE;
+}
+
 /* Initializes context and preferences, then loads key manager */
 int
 main (int argc, char **argv)
 {
     GnomeProgram *program = NULL;
-	SeahorseContext *sctx;
+    SeahorseOperation *op = NULL;
     GtkWindow* win;
     int ret = 0;
 
-#ifdef ENABLE_NLS	
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
+    seahorse_secure_memory_init (65536);
+    
+    /* We need to drop privileges completely for security */
+#if defined(HAVE_SETRESUID) && defined(HAVE_SETRESGID)
+
+    /* Not in header files for all OSs, even where present */
+    int setresuid(uid_t ruid, uid_t euid, uid_t suid);
+    int setresgid(gid_t rgid, gid_t egid, gid_t sgid);
+  
+    if (setresuid (getuid (), getuid (), getuid ()) == -1 ||
+        setresgid (getgid (), getgid (), getgid ()) == -1)
+#else
+    if (setuid (getuid ()) == -1 || setgid (getgid ()) == -1)
+#endif
+        g_error (_("couldn't drop privileges properly"));
+    
+#ifdef ENABLE_NLS
+    bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
 #endif
 
-	g_message("init gpgme version %s", gpgme_check_version(NULL));
+    g_message("init gpgme version %s", gpgme_check_version(NULL));
 
-#ifdef ENABLE_NLS	
-	gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
-	gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
+#ifdef ENABLE_NLS
+    gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
+    gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
 #endif
 
     program = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
                     GNOME_PARAM_POPT_TABLE, options,
-                    GNOME_PARAM_HUMAN_READABLE_NAME, _("GPG Keys Manager"),
+                    GNOME_PARAM_HUMAN_READABLE_NAME, _("Encryption Key Manager"),
                     GNOME_PARAM_APP_DATADIR, DATA_DIR, NULL);
 
-    sctx = seahorse_context_new ();
-    seahorse_context_load_keys (sctx, FALSE);
+#ifdef FATAL_MESSAGES 
+    g_log_set_always_fatal (G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL | 
+                            G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
+#endif
+    
+    /* Insert Icons into Stock */ 
+    seahorse_gtkstock_init();
+    
+    /* Make the default SeahorseContext */
+    seahorse_context_new (TRUE, 0);
+    op = seahorse_context_load_local_keys (SCTX_APP());
 
     if (mode != MODE_NONE) {
         gchar **uris = NULL;
+        
+        /* The op frees itself */
+        g_object_unref (op);
        
         /* Load up all our arguments */
         uris = read_uri_arguments(program);
@@ -446,22 +477,22 @@ main (int argc, char **argv)
         if(uris && uris[0]) {
             switch (mode) {
             case MODE_IMPORT:
-                ret = do_import (sctx, (const gchar**)uris);
+                ret = do_import ((const gchar**)uris);
                 break;
             case MODE_ENCRYPT:
-                ret = do_encrypt (sctx, (const gchar**)uris);
+                ret = do_encrypt ((const gchar**)uris);
                 break;
             case MODE_SIGN:
-                ret = do_sign (sctx, (const gchar**)uris);
+                ret = do_sign ((const gchar**)uris);
                 break;
             case MODE_ENCRYPT_SIGN:
-                ret = do_encrypt_sign (sctx, (const gchar**)uris);
+                ret = do_encrypt_sign ((const gchar**)uris);
                 break;
             case MODE_DECRYPT:
-                ret = do_decrypt (sctx, (const gchar**)uris);
+                ret = do_decrypt ((const gchar**)uris);
                 break;
             case MODE_VERIFY:
-                ret = do_verify (sctx, (const gchar**)uris);
+                ret = do_verify ((const gchar**)uris);
                 break;
             default:
                 g_assert_not_reached ();
@@ -470,15 +501,18 @@ main (int argc, char **argv)
                 
             g_strfreev (uris);
         }
+        
+        g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)check_dialogs, NULL, NULL);
+        gtk_main ();
 
     } else { 
-        win = seahorse_key_manager_show (sctx);
+        win = seahorse_key_manager_show (op);
         g_signal_connect_after (G_OBJECT (win), "destroy", gtk_main_quit, NULL);
-	    gtk_main ();
-	}
+        gtk_main ();
+    }
     
     if (gnome_vfs_initialized ())
         gnome_vfs_shutdown ();
     
-	return ret;
+    return ret;
 }
